"""
"""
import imp
import os.path
import sys


def add_search_path(*path_tokens):
    """
    Adds a new search path from where modules can be loaded. 
    
    This function is provided for test applications to add locations to the search path, so any required functionality
    can be loaded. It helps keeping the step implementation modules simple by placing the bulk of the implementation in
    separate utility libraries. This function can also be used to add the application being tested to the path, so its
    functionality can be made available for testing.

    :param arglist path_tokens: 
        Variable list of path tokens that is joined to create the full, absolute path to be added.
    """
    full_path = os.path.join(*path_tokens)
    if full_path not in sys.path:
        sys.path.insert(0, os.path.abspath(full_path))



def load_script(filename):
    """
    Loads a python script as a module.

    This function is provided to allow applications to load a Python module by its file name.

    :param string filename:
        Name of the python file to be loaded as a module.

    :return:
        A |Python|_ module loaded from the specified file.
    """
    path, module_name, ext = _extract_script_components(filename)
    add_search_path(path)
    return _load_module(module_name)


def _extract_script_components(filename):
    path = os.path.dirname(filename)
    base_name = os.path.basename(filename)
    module_name, ext = os.path.splitext(base_name)
    return path, module_name, ext


def _load_module(module_name):
    file = None
    try:
        file, pathname, description = imp.find_module(module_name)
        module = imp.load_module(module_name, file, pathname, description)
        return module
    finally:
        if file:
            file.close()





"""
"""
import glob
import logging
import os

class FileFinder(object):
    """
    """

    def __init__(self, sourcedir, pattern, recursive):
        self.sourcedir = sourcedir
        self.pattern = pattern
        self.recursive = recursive
        self.matches = None


    def find(self):
        """
        """
        self.matches = []
        if self.recursive:
            self.matches = self._search_recursive(self.sourcedir, self.pattern)
        else:
            fullpattern = os.path.join(self.sourcedir, self.pattern)
            self.matches = self._search_matches(fullpattern)
        return self.matches



    def _search_recursive(self, path, pattern):
        result = []
        for path, dirs, files in os.walk(self.sourcedir):
            fullpattern = os.path.join(path, self.pattern)
            result.extend(self._search_matches(fullpattern))
        return result

    def _search_matches(self, fullpattern):
        return glob.glob(fullpattern)



def delete_files_in(all_files):
    """
    """
    for f in all_files:
        os.remove(f)
        logging.info(f + ' deleted')



"""
sleep
-----

This task allows to set the bolt process to sleep for the specified duration. 
If no duration is specified, the process will sleep forever. This task can be 
used to wait for a subprocess to start in cases where the subprocess might take 
some time to initialize. You can also use it to wait forever in case that an 
independent subprocess needs to be kept running in the background./

..  code-block:: python

    config = {
        'sleep': {
            'duration': 10      # Waits 10 seconds.
        }
    }
"""
import logging
import time

import bolt.utils as btutils


class SleepTask(object):

    def __call__(self, **kwargs):
        self.config = kwargs.get('config')
        self.duration = self.config.get('duration') or btutils.FOREVER
        if self.duration == btutils.FOREVER:
            self._sleep_forever()
        else:
            self._sleep(self.duration)


    def _sleep(self, seconds):
        time.sleep(seconds)


    def _sleep_forever(self):
        while True: pass



def register_tasks(registry):
    registry.register_task('sleep', SleepTask())




"""
setup
-----

The ``setup`` task provides an automation hook to execute ``setup.py``
commands and options inside of Bolt. The task, in its simplest form,
assumes a default ``setup.py`` in the current working directory and
uses a ``build`` command as a default if no configuration is provided.

The task configuration allows spcifying a setup script, which by 
default will be set to ``setup.py`` if no script is specified, a valid
command, and it command arguments. The following example shows how to
configure the task. ::

    config = {
		'setup':{
			'script': 'special_setup.py',
			'command': 'install',
			'options': {
				'verbose': True,
				'dry-run': True
			}
		}
    }
"""
import distutils.core as dcore
import logging

import bolt.utils as utilities

DEFAULT_ARGUMENTS = ['build']
DEFAULT_SETUP_SCRIPT = 'setup.py'

class _SetupArgumentGenerator(utilities.CommonCommandAndArgumentsGenerator):

    def __init__(self):
        return super(_SetupArgumentGenerator, self).__init__(DEFAULT_ARGUMENTS)



class ExecuteSetupTask(object):
    
    def __call__(self, **kwargs):
        logging.info('Executing Setup')
        config = kwargs.get('config')
        self.setup_script = config.get('script')
        if self.setup_script:
            config['script'] = False
        else:
            self.setup_script = DEFAULT_SETUP_SCRIPT
        generator = _SetupArgumentGenerator()
        self.args = generator.generate_from(config)
        self._execute_setup()


    def _execute_setup(self):
        dcore.run_setup(self.setup_script, self.args)



def register_tasks(registry):
    registry.register_task('setup', ExecuteSetupTask())



import unittest

import bolt.errors as bterrors
import bolt._btregistry as registry

class TestTaskRegistry(unittest.TestCase):

    def setUp(self):
        self.subject = registry.TaskRegistry()

    def test_can_register_callable_as_task(self):
        task_name = 'test'
        self.subject.register_task(task_name, self.test_callable)
        actual = self.subject.get(task_name)

        self.assertEqual(actual, self.test_callable)


    def test_can_register_list_of_task_names(self):
        task_name = 'list'
        task_list = ['task1', 'task2', 'task3']
        self.subject.register_task(task_name, task_list)
        actual_list = self.subject.get(task_name)

        self.assertSequenceEqual(task_list, actual_list)


    def test_fails_if_task_is_not_callable_or_list(self):
        with self.assertRaises(bterrors.InvalidTaskError):
            self.subject.register_task("invalid", 1)


    def test_raises_if_list_contains_anything_other_than_strings(self):
        with self.assertRaises(bterrors.InvalidTaskError):
            self.subject.register_task("invalid", ['foo', 'bar', 3])


    def test_allows_specifying_subtask_returing_correct_callable(self):
        task_name = 'test'
        subtask_name = 'test.sub'
        self.subject.register_task(task_name, self.test_callable)
        actual = self.subject.get(subtask_name)

        self.assertEqual(actual, self.test_callable)



    def test_callable(self):
        pass


if __name__ == '__main__':
    unittest.main()



"""
.. _task-delete-files:

delete-files
------------

This task deletes files matching a specified ``pattern`` found in a specified
``sourcedir``. A ``recursive`` flag can be specified to search also in 
sub-directories of ``sourcedir``.

The ``pattern`` specified follows the matching rules of the |python|_ standard
library ``glob.glob()`` function.

The following example configures the ``delete-file`` task to delete all the
files in a ``tmp`` directory located at the project root, and all its
sub-directories::

    config = {
        'delete-files': {
            'sourcedir': './tmp',
            'pattern': '*.*',
            'recursive': True
        }
    }

The ``sourcedir`` configuration option indicates the directory to search for
file matches. This option is required.

The ``pattern`` option specifies the matching pattern to find files. This
option is required.

The ``recursive`` option indicates if sub-directories should be searched for
matches. This option is optional and has a value of ``False`` by default.


.. _task-delete-pyc:

delete-pyc
----------

Searches for ``.pyc`` in the specified directory and deletes them. The task
allows to recursively search sub-directories for ``.pyc`` files.

The following example shows how to configure the task to recursively delete
files from a source directory and its sub-directories::

    config = {
        'delete-pyc': {
            'sourcedir': './source',
            'recursive': True
        }
    }

The ``sourcedir`` option specifies the directory to search for ``.pyc`` files.
This option is required.

The ``recursive`` option indicates if sub-directories should be searched for
matches. This option is optional and has a value of ``False`` by default. 
"""
import glob
import logging
import os

import bolt.errors as bterrors
import bolt.utils as utilities

class DeleteFilesTask(object):

    def __call__(self, **kwargs):
        self._set_valid_configuration(kwargs.get('config'))
        self._execute_delete()


    def _set_valid_configuration(self, config):
        if not config:
            raise bterrors.InvalidConfigurationError('Configuration cannot be empty')
        self.sourcedir = config.get('sourcedir')
        if not self.sourcedir:
            raise bterrors.InvalidConfigurationError('Source directory not specified')
        self.pattern = config.get('pattern')
        if not self.pattern:
            raise bterrors.InvalidConfigurationError('File pattern not specified')
        self.recursive = config.get('recursive')


    def _execute_delete(self):
        logging.info('Deleting {pat} from {srcdir}.'.format(pat=self.pattern, srcdir=self.sourcedir))
        finder = utilities.FileFinder(self.sourcedir, self.pattern, self.recursive)
        matches = finder.find()
        utilities.delete_files_in(matches)



class DeletePycTask(DeleteFilesTask):

    def __call__(self, **kwargs):
        config = kwargs.get('config')
        config['pattern'] = '*.pyc'
        logging.debug('Delete pattern set to ' + config['pattern'])
        return super(DeletePycTask, self).__call__(**kwargs)



def register_tasks(registry):
    registry.register_task('delete-files', DeleteFilesTask())
    registry.register_task('delete-pyc', DeletePycTask())   




import logging
import os.path
import unittest

import bolt._btapp as btapp
import bolt.errors as bterror
import bolt._btregistry as btregistry


class TestBoltApplication(unittest.TestCase):
    
    def setUp(self):
        self.options = OptionsDummy()
        self.registry = btregistry.TaskRegistry()
        self.config_manager = ConfigManagerDummy()
        self.application = btapp.BoltApplication(self.options, self.registry, self.config_manager)
        self.default_application = BoltApplicationSpy()
        return super(TestBoltApplication, self).setUp()

    
    def test_uses_specified_options(self):
        self.assertIs(self.application.options, self.options)


    def test_uses_default_options_if_none_specified(self):
        self.assertIs(self.default_application.options, self.default_application.default_options)


    def test_uses_specified_registry(self):
        self.assertIs(self.application.registry, self.registry)


    def test_uses_default_registry_if_none_specified(self):
        self.assertIs(self.default_application.registry, self.default_application.default_registry)


    def test_uses_specified_configuration_manager(self):
        self.assertIs(self.application.config_manager, self.config_manager)


    def test_delays_initialization_of_configuration_manager_if_not_specified(self):
        self.assertIsNone(self.default_application.config_manager)


    def test_runs_task_specified(self):
        self.setup_tasks()
        self.application.run_task('successful_task')
        self.assertTrue(self.successful_task_executed)


    def test_continues_on_error_if_specified(self):
        self.setup_tasks()
        self.application.options.continue_on_error = True
        self.application.run_task('continue')
        self.assertTrue(self.successful_task_executed)


    def test_can_override_continue_on_error(self):
        self.setup_tasks()
        self.application.run_task('continue', True)
        self.assertTrue(self.successful_task_executed)


    def test_raises_system_exit_if_no_continue_on_error_set(self):
        with self.assertRaises(bterror.TaskError):
            self.setup_tasks()
            self.application.run_task('continue')
            self.assertFalse(self.successful_task_executed)


    def test_tears_down_tasks_at_the_end(self):
        self.setup_tasks()
        self.application.run_task('tearable')
        self.assertTrue(self.tearable_task.tear_down_invoked)


    def test_tears_down_even_if_exception(self):
        try:
            self.setup_tasks()
            self.application.run_task('insure_teardown')
        except bterror.TaskError:
            self.assertTrue(self.tearable_task.tear_down_invoked)


    def test_executes_specified_task(self):
        self.setup_tasks()
        self.application.options.task = 'successful_task'
        self.application.run()
        self.assertTrue(self.successful_task_executed)


    def test_standard_tasks_are_registered(self):
        self.assert_registered('conttest')
        self.assert_registered('coverage')
        self.assert_registered('delete-files')
        self.assert_registered('delete-pyc')
        self.assert_registered('mkdir')
        self.assert_registered('nose')
        self.assert_registered('pip')
        self.assert_registered('setup')
        self.assert_registered('shell')
        self.assert_registered('sleep')


    def setup_tasks(self):
        self.tearable_task = TearableTask()
        self.application.registry.register_task('tearable', self.tearable_task)
        self.application.registry.register_task('successful_task', self.successful_task)
        self.application.registry.register_task('failed_task', self.failed_task)
        self.application.registry.register_task('continue', ['failed_task', 'successful_task'])
        self.application.registry.register_task('insure_teardown', ['tearable', 'failed_task'])


    def assert_registered(self, task_name):
        task = self.application.registry.get(task_name)
        self.assertIsNotNone(task)
        


    def successful_task(self, **kwargs):
        self.successful_task_executed = True
        return 0


    def failed_task(self, **kwargs):
        raise bterror.TaskError()





class OptionsDummy(object):
    
    def __init__(self):
        self.log_level = logging.CRITICAL
        self.log_file = None
        self.continue_on_error = False


class ConfigManagerDummy(object):
    
    def get(self, task):
        return {}


class BoltApplicationSpy(btapp.BoltApplication):
    
    def __init__(self):
        self.default_options = OptionsDummy()
        self.default_registry = None
        self.default_config_manager = ConfigManagerDummy()
        return super(BoltApplicationSpy, self).__init__()


    def _get_default_options(self):
        return self.default_options


    def _get_default_registry(self):
        registry = super(BoltApplicationSpy, self)._get_default_registry()
        self.default_registry = registry
        return self.default_registry


    def _get_default_config_manager(self):
        return self.default_config_manager


    def _get_file_logger_handler(self, filename):
        return logging.StreamHandler()



class TearableTask(object):
    
    def __call__(self, **kwargs):
        pass 


    def tear_down(self):
        self.tear_down_invoked = True



class LoggerDummy(object):
    
    def __init__(self):
        self.handler_count = 0


    def setLevel(self, level):
        self.level = level


    def addHandler(self, handler):
        self.handler_count += 1




class TestBoltFile(unittest.TestCase):
    

    def test_loads_specified_file(self):
        bfile = os.path.abspath('a_bolt_file.py')
        boltfile = BoltFileSpy(bfile)
        self.assertEqual(boltfile.filename, bfile)


    def test_returns_config_from_module(self):
        boltfile = BoltFileSpy('a_bolt_file.py')
        self.assertEqual(boltfile.config, boltfile.expected_module.config)



class BoltFileSpy(btapp.BoltFile):
    
    def _load(self):
        self.expected_module = BoltModuleDouble()
        return self.expected_module



class BoltModuleDouble(object):
    
    @property
    def config(self):
        return {}



if __name__=="__main__":
    unittest.main()


"""
Exposes classes related to the supported options for Bolt.
"""
import argparse
import logging

class Commands(object):
    """
    Defines the supported commands.
    """
    EXECUTE = 'execute'
    HELP = 'help'
    VERSION = 'version'


class Default(object):
    """
    Defines default values for options.
    """
    COMMAND = Commands.EXECUTE
    TASK = 'default'
    BOLTFILE = 'boltfile.py'
    LOG_LEVEL = logging.INFO
    LOG_FILE = None
    CONTINUE_ON_ERROR = False


class OptionSwitch(object):
    """
    Defines values for the command line switches.
    """
    TASK = 'task'
    BOLTFILE_SHORT = '-b'
    BOLTFILE_LONG = '--bolt-file'
    LOG_LEVEL_SHORT = '-v'
    LOG_LEVEL_LONG = '--log-level'
    LOG_FILE_SHORT = '-o'
    LOG_FILE_LONG = '--log-file'
    CONTINUE_ON_ERROR_LONG = '--continue-on-error'
    VERSION_LONG = '--version'



class Options(object):
    """
    Exposes application options that are initialized from the command line or
    by specifying a parameters list. 
    """

    def __init__(self, opt_list=None):
        self._command = Commands.EXECUTE
        self._initialize_options(opt_list)
    
    @property
    def command(self):
        """
        Returns the command to be executed.
        """
        return self._command


    @property
    def task(self):
        """
        Returns the task to be executed or the default task if none specified.
        """
        return self._options.task


    @property
    def bolt_file(self):
        """
        Returns the bolt file to use or the default bolt file if none specified.
        """
        return self._options.bolt_file


    @property
    def log_level(self):
        """
        Returns the log level to use or the default log level if none specified.
        """
        return self._options.log_level


    @property
    def log_file(self):
        """
        Returns the log file to use or None if no file is specified.
        """
        return self._options.log_file


    @property
    def continue_on_error(self):
        """
        Returns whether to continue executing tasks when errors are found. The
        default is false.
        """
        return self._options.continue_on_error


    def _initialize_options(self, opt_list=None):
        parser = self._build_parser()
        self._options = parser.parse_args(opt_list)
        self._marshall_options()


    def _build_parser(self):
        parser = argparse.ArgumentParser()
        parser.add_argument(OptionSwitch.TASK, nargs='?', default=Default.TASK, 
            help='Task to be executed. It uses "default" if not specified')
        parser.add_argument(OptionSwitch.BOLTFILE_SHORT, OptionSwitch.BOLTFILE_LONG, default=Default.BOLTFILE,
            help='Bolt file containing the configuration and task. The default is "boltfile.py"')
        parser.add_argument(OptionSwitch.LOG_LEVEL_SHORT, OptionSwitch.LOG_LEVEL_LONG, default=Default.LOG_LEVEL,
            help='Specifies logging level.')
        parser.add_argument(OptionSwitch.LOG_FILE_SHORT, OptionSwitch.LOG_FILE_LONG, default=None, 
            help='Log file to write the output.')
        parser.add_argument(OptionSwitch.CONTINUE_ON_ERROR_LONG, action='store_true', 
            help='Continue executing even if errors are found')
        parser.add_argument(OptionSwitch.VERSION_LONG, action='store_true',
            help='Shows the the application version')
        return parser


    def _marshall_options(self):
        self._marshall_command()
        self._marshall_log_level()


    def _marshall_command(self):
        if self._options.version:
            self._command = Commands.VERSION


    def _marshall_log_level(self):
        if self._options.log_level == Default.LOG_LEVEL:
            return
        lc_level = self._options.log_level.lower()
        if lc_level in ['', 'n', 'notset']:
            self._options.log_level = logging.NOTSET
        elif lc_level in ['debug', 'd', 'dbg']:
            self._options.log_level = logging.DEBUG
        elif lc_level in ['info', 'i', 'information']:
            self._options.log_level = logging.INFO
        elif lc_level in ['warning', 'warn', 'w']:
            self._options.log_level = logging.WARNING
        elif lc_level in ['error', 'err', 'e']:
            self._options.log_level = logging.ERROR
        elif lc_level in ['critical', 'crit', 'c']:
            self._options.log_level = logging.CRITICAL

        


"""
Classes to help processing configurations.
"""

TO_ENABLE_FLAG = True
TO_DISABLE_FLAG = False


def append_with_equal(arg_list, formatted_option, value):
    if value is TO_ENABLE_FLAG:
        arg_list.append(formatted_option)
    elif value is not TO_ENABLE_FLAG:
        formatted_argument = '{option}={value}'.format(option=formatted_option, value=value)
        arg_list.append(formatted_argument)



def append_as_tokens(arg_list, formatted_option, value):

    arg_list.append(formatted_option)
    if value is not TO_ENABLE_FLAG:
        arg_list.append(value)


class ArgumentsGenerator(object):
    
    def __init__(self, default_args, append_callback=None):
        self.default_arguments = default_args
        self.append_callback = append_callback or append_as_tokens


    def generate_from(self, config):
        self.args = []
        if not config:
            return self.default_arguments
        self.config = config
        self._convert_config_to_arguments()
        return self.args


    def _convert_config_to_arguments(self):
        self.options = self.config.get('options')
        if self.options:
            {self._push_as_arguments(option, value) for option, value in self.options.items()}       
        
        

    def _push_as_arguments(self, option, value):
        if value is not TO_DISABLE_FLAG:
            self._do_push(option, value)        


    def _do_push(self, option, value):
        formatted_option = self._format_option(option)
        self.append_callback(self.args, formatted_option, value)
        
        

    def _format_option(self, option):
        if len(option) == 1:
            fmt_str = '-{option}'
        else:
            fmt_str = '--{option}'
        formatted_option = fmt_str.format(option=option)
        return formatted_option


    





class CommonCommandAndArgumentsGenerator(ArgumentsGenerator):

    
    def _convert_config_to_arguments(self):
        self.command = self.config.get('command')
        self.args.append(self.command)
        return super(CommonCommandAndArgumentsGenerator, self)._convert_config_to_arguments()


"""
This is the main module that exposes the required functions to use bolt in your applications.
"""
import logging
import sys

import bolt._btapp as btapp
import bolt._btoptions as btoptions
import bolt.utils.log as btlog


def register_module_tasks(module):
    """
    """
    app = btapp.get_application()
    app.registry.register_module_tasks(module)


def register_task(name, task):
    """
    """
    app = btapp.get_application()
    app.registry.register_task(name, task)


def run_task(task_name, continue_on_error=None):
    continue_on_error = continue_on_error or False
    app = btapp.get_application()
    app.run_task(task_name, continue_on_error)


def run():
    """
    Entry point for the `bolt` executable.
    """
    options = btoptions.Options()
    btlog.initialize_logging(options.log_level, options.log_file)
    app = btapp.get_application()
    app.run()

    

if __name__=="__main__":
    run()



"""
.. _task-nose:

nose
----

Executes unit tests using nose and nosetests as the unit test runner. The task
allows to specify the directory where the tests are located through the ``directory``
parameter and supports all the arguments available in the installed version 
of nosetests::

    config = {
        'nose': {
            'directory': 'test/unit',
            'options': {
                'xunit-file': 'output/unit_tests.xml'
                'with-coverage': True,
                'cover-erase': True,
                'cover-package': 'mypackage',
                'cover-html': True,
                'cover-html-dir': 'output/coverage',
            }
    }
"""
import logging
import os.path
import subprocess as sp

import bolt.errors as bterror
import bolt.utils as utilities


DEFAULT_DIR = './'
DEFAULT_ARGUMENTS = [DEFAULT_DIR]


class NoseError(bterror.TaskError):

    def __init__(self, nose_code):
        msg = "nose exited with code {code}".format(code=nose_code)
        super(NoseError, self).__init__(msg)


class _NoseArgumentGenerator(utilities.ArgumentsGenerator):

    def __init__(self):
        return super(_NoseArgumentGenerator, self).__init__(DEFAULT_ARGUMENTS, utilities.append_with_equal)


    def _convert_config_to_arguments(self):
        self.args.append('nosetests')
        super(_NoseArgumentGenerator, self)._convert_config_to_arguments()
        directory = self.config.get('directory') or DEFAULT_DIR
        directory = os.path.abspath(directory)
        self.args.append(directory)



class ExecuteNoseTask(object):
    
    def __call__(self, **kwargs):
        logging.info('Executing nose')
        config = kwargs.get('config')
        generator = _NoseArgumentGenerator()
        self.args = generator.generate_from(config)
        logging.debug('Arguments: ' + repr(self.args))
        self._execute_nose()


    def _execute_nose(self):
        result = sp.call(self.args)
        if result != 0:
            raise NoseError(result)




def register_tasks(registry):
    registry.register_task('nose', ExecuteNoseTask())




"""
"""
import logging

class TaskRunner(object):
    """
    """

    def __init__(self, config, registry, continue_on_error):
        self._config = config
        self._registry = registry
        self._continue_on_error = continue_on_error
        self._context = {}
        self._script = None
        self._executed_operations = None


    def run(self, task):
        self._build(task)
        for task in self._script:
            self._try_run_task(task)


    def tear_down(self):
        for operation in self._executed_operations:
            if hasattr(operation, 'tear_down'):
                operation.tear_down()

    def _build(self, task_name):
        self._script = []
        self._executed_operations = []
        self._build_task(task_name)


    def _build_task(self, task_name):
        task_operation = self._registry.get(task_name)
        if callable(task_operation):
            task_config = self._config.get(task_name)
            requested_task = (task_operation, task_config)
            self._script.append(requested_task)
        else:
            for subtask in task_operation:
                self._build_task(subtask)


    def _try_run_task(self, task):
        try:
            self._run_task(task)
        except Exception as ex:
            logging.error(ex)
            if not self._continue_on_error:
                raise ex


    def _run_task(self, task):
        operation, config = task
        result = operation(config=config, context=self._context)
        self._executed_operations.append(operation)



import logging

def initialize_logging(log_level, log_file=None, logger=None):
    logger = logger or logging.getLogger()
    logger.setLevel(log_level)
    # Console logging
    handler = logging.StreamHandler()
    formatter = logging.Formatter("%(levelname)s - %(message)s")
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    # File logging if specified.
    if log_file:
        handler = logging.FileHandler(log_file, 'w')
        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
        handler.setFormatter(formatter)
        logger.addHandler(handler)


"""
"""
import logging
import unittest

import bolt._btoptions as btoptions

class TestOptions(unittest.TestCase):
    
    def setUp(self):
        # If we don't initialize it from an empty options list, it parses
        # the arguments to nosetests and the test fail. The actual code using
        # it doesn't need to do it because it will parse the arguments to 
        # bolt.
        self.default_options = btoptions.Options([])
        return super(TestOptions, self).setUp()


    def test_default_command_is_properly_initialized(self):
        self.assertEqual(self.default_options.command, btoptions.Default.COMMAND)


    def test_command_is_version_if_switch_specified(self):
        self.given(btoptions.OptionSwitch.VERSION_LONG)
        self.assertEqual(self.options.command, btoptions.Commands.VERSION)


    def test_default_task_is_properly_initialized(self):
        self.assertEqual(self.default_options.task, btoptions.Default.TASK)


    def test_returns_correct_task_if_specified(self):
        task = 'a_task'
        self.given(task)
        self.assertEqual(self.options.task, task)


    def test_default_boltfile_is_properly_initialized(self):
        self.assertEqual(self.default_options.bolt_file, btoptions.Default.BOLTFILE)


    def test_sets_boltfile_with_long_switch(self):
        boltfile = 'a_bolt_file.py'
        self.given(btoptions.OptionSwitch.BOLTFILE_LONG, boltfile)
        self.assertEqual(self.options.bolt_file, boltfile)


    def test_sets_boltfile_with_short_switch(self):
        boltfile = 'a_bolt_file.py'
        self.given(btoptions.OptionSwitch.BOLTFILE_SHORT, boltfile)
        self.assertEqual(self.options.bolt_file, boltfile)


    def test_default_log_level_is_properly_initialized(self):
        self.assertEqual(self.default_options.log_level, btoptions.Default.LOG_LEVEL)


    def test_sets_log_level_with_long_switch(self):
        log_level = 'error'
        self.given(btoptions.OptionSwitch.LOG_LEVEL_LONG, log_level)
        self.assertEqual(self.options.log_level, logging.ERROR)


    def test_sets_log_level_with_short_switch(self):
        log_level = 'debug'
        self.given(btoptions.OptionSwitch.LOG_LEVEL_SHORT, log_level)
        self.assertEqual(self.options.log_level, logging.DEBUG)


    def test_converts_correctly_from_log_level_string_to_logging_level(self):
        # NOTSET
        self.verify_log_level('', logging.NOTSET)
        self.verify_log_level('n', logging.NOTSET)
        self.verify_log_level('notset', logging.NOTSET)
        # DEBUG
        self.verify_log_level('d', logging.DEBUG)
        self.verify_log_level('dbg', logging.DEBUG)
        self.verify_log_level('debug', logging.DEBUG)


    def test_default_log_file_is_properly_initialized(self):
        self.assertEqual(self.default_options.log_file, btoptions.Default.LOG_FILE)


    def test_sets_the_log_file_with_long_switch(self):
        log_file = 'log.txt'
        self.given(btoptions.OptionSwitch.LOG_FILE_LONG, log_file)
        self.assertEqual(self.options.log_file, log_file)


    def test_sets_the_log_file_with_short_switch(self):
        log_file = 'log.txt'
        self.given(btoptions.OptionSwitch.LOG_FILE_SHORT, log_file)
        self.assertEqual(self.options.log_file, log_file)


    def test_continue_on_error_is_properly_initialized(self):
        self.assertEqual(self.default_options.continue_on_error, btoptions.Default.CONTINUE_ON_ERROR)


    def test_sets_continue_on_error_with_long_switch(self):       
        self.given(btoptions.OptionSwitch.CONTINUE_ON_ERROR_LONG)
        self.assertTrue(self.options.continue_on_error)


    def given(self, *args):
        self.options = btoptions.Options(args)

    
    def verify_log_level(self, str_level, expected):
        self.given(btoptions.OptionSwitch.LOG_LEVEL_LONG, str_level)
        self.assertEqual(self.options.log_level, expected)



if __name__=="__main__":
    unittest.main()



"""
"""
import logging
import os.path

import bolt._btconfig as btconfig
import bolt._btoptions as btoptions
import bolt._btregistry as btregistry
import bolt._btrunner as btrunner
from bolt._btutils import load_script

# Standard task modules.
import bolt.tasks.bolt_pip as bolt_pip
import bolt.tasks.bolt_delete_files as bolt_delete_files
import bolt.tasks.bolt_setup as bolt_setup
import bolt.tasks.bolt_shell as bolt_shell
import bolt.tasks.bolt_conttest as bolt_conttest
import bolt.tasks.bolt_nose as bolt_nose
import bolt.tasks.bolt_mkdir as bolt_mkdir 
import bolt.tasks.bolt_coverage as bolt_coverage
import bolt.tasks.bolt_sleep as bolt_sleep


class BoltApplication(object):
    """
    """

    def __init__(self, options=None, registry=None, config_manager=None):
        self._options = options or self._get_default_options()
        self._registry = registry or self._get_default_registry()
        self._config_manager = config_manager
        self._register_standard_tasks()


    @property
    def config_manager(self):
        """
        """
        return self._config_manager

    
    @property
    def options(self):
        """
        """
        return self._options


    @property
    def registry(self):
        """
        """
        return self._registry


    def run(self):
        """
        """
        self.run_task(self.options.task)


    def run_task(self, task_name, continue_on_error=None):
        """
        """
        continue_on_error = continue_on_error or self.options.continue_on_error
        if not self._config_manager:
            self._config_manager = self._get_default_config_manager()
        runner = btrunner.TaskRunner(self.config_manager, self.registry, continue_on_error)
        try:
            runner.run(task_name)
        finally:
            runner.tear_down()


    def _get_default_options(self):
        return btoptions.Options()


    def _get_default_registry(self):
        return btregistry.TaskRegistry()


    def _get_default_config_manager(self):
        bf = BoltFile(self.options.bolt_file)
        return btconfig.ConfigurationManager(bf.config)


    def _register_standard_tasks(self):
        self.registry.register_module_tasks(bolt_conttest)
        self.registry.register_module_tasks(bolt_coverage)
        self.registry.register_module_tasks(bolt_delete_files)
        self.registry.register_module_tasks(bolt_mkdir)
        self.registry.register_module_tasks(bolt_nose)
        self.registry.register_module_tasks(bolt_pip)
        self.registry.register_module_tasks(bolt_setup)
        self.registry.register_module_tasks(bolt_shell)
        self.registry.register_module_tasks(bolt_sleep)


    def _register_module(self, module):
        module.register_tasks(self.registry)



class BoltFile(object):
    """
    """
    
    def __init__(self, bolt_file_name):
        self.filename = os.path.abspath(bolt_file_name)
        self._bolt_file_module = self._load()


    def _load(self):
        return load_script(self.filename)


    @property
    def config(self):
        """
        """
        return self._bolt_file_module.config


_application_instance = None

def get_application(options=None):
    global _application_instance
    if not _application_instance:
        _application_instance = BoltApplication(options=options)
    return _application_instance




"""
mkdir
-----

Creates the directory specified, including intermediate directories, if they
do not exist::

    config = {
        'mkdir': {
            'directory': 'several/intermediate/directories'
        }
    }
"""
import logging
import os


class ExecuteMKDir(object):
    

    def __call__(self, **kwargs):
        config = kwargs.get('config')
        self.directory = config.get('directory')
        logging.debug('Creating directory: {directory}'.format(directory=self.directory))
        if not os.path.exists(self.directory):
            self._create_directories()


    def _create_directories(self):
        os.makedirs(self.directory)


def register_tasks(registry):
    registry.register_task('mkdir', ExecuteMKDir())
    


"""
coverage
--------

This task allows to measure code coverage of source code executed by the specified
task. The following is a sample configuration:

..  code-block:: python

    config = {
        'coverage': {
            'task': 'registered-task',
            'include': './directory/for/source',
            'output': 'directory/to/write/results'
        }
    }

The specified task must exercise the code that will be measured.
"""
import logging
import bolt


class ExecuteCoverage(object):
    
    def __call__(self, **kwargs):
        self.config = kwargs.get('config')
        self.task_name = self.config.get('task')
        self.include_dir = self.config.get('include')
        self.out_dir = self.config.get('output')
        logging.info('Code coverage for {task}. Output at {directory}'.format(task=self.task_name, directory=self.out_dir))
        self.execute_with_coverage()


    def execute_with_coverage(self):
        import coverage.control as cov
        controller = cov.Coverage(auto_data=False, branch=True, source=self.include_dir)
        controller.start()
        bolt.run_task(self.task_name)
        controller.stop()
        controller.html_report(directory=self.out_dir)



def register_tasks(registry):
    registry.register_task('coverage', ExecuteCoverage())
    


import unittest

import bolt._btconfig as config
import bolt.errors as bterror
import bolt._btregistry as registry

import bolt._btrunner as runner


class TestTaskRunner(unittest.TestCase):

    def setUp(self):
        self.setUpRegistry()
        self.setUpConfig()
        self.subject = runner.TaskRunner(self.config_mgr, self.registry, False)
        self.executed_tasks = []
        return super(TestTaskRunner, self).setUp()


    def setUpRegistry(self):
        self.registry = registry.TaskRegistry()
        self.registry.register_task('task_1', self.task_1_callback)
        self.registry.register_task('task_2', self.task_2_callback)
        self.registry.register_task('invalid', self.invalid_task)
        self.registry.register_task('multi_task', ['task_1', 'task_2'])
        self.registry.register_task('configurable', self.configurable_task)
        self.registry.register_task('failing_task', self.failing_task)
        self.registry.register_task('multi_task_with_failures', ['task_1', 'failing_task', 'task_2'])
        self.tear_down_task = TearDownTask()
        self.registry.register_task('has_tear_down', self.tear_down_task)
        self.registry.register_task('partially_executed', ['failing_task', 'has_tear_down'])


    def setUpConfig(self):
        self.config = {
                'configurable': {
                    'param': 'value'
                }
            }
        self.config_mgr = config.ConfigurationManager(self.config)
    

    def test_runs_the_specified_task(self):
        self.given('task_1')
        self.expect_executed('task_1')


    def test_configuration_is_passed_to_task_by_named_parameter(self):
        self.given('task_2')
        self.expect_executed('task_2')


    def test_raises_if_task_definition_does_not_meet_spec(self):
        with self.assertRaises(Exception):
            self.given('invalid')


    def test_executes_all_tasks_in_multi_task(self):
        self.given('multi_task')
        self.expect_executed('task_1')
        self.expect_executed('task_2')


    def test_task_is_executed_with_correct_configuration(self):
        self.given('configurable')
        self.expect_executed('configurable')


    def test_build_is_enforced_before_run(self):
        with self.assertRaises(Exception):
            subject = runner.TaskRunner(self.config_mgr, self.registry, False)
            subject.run()


    def test_build_fails_if_a_task_is_not_registered(self):
        with self.assertRaises(Exception):
            self.subject.build('inexistent')


    def test_exits_if_task_does_not_return_zero(self):
        with self.assertRaises(bterror.TaskError):
            self.given('failing_task')


    def test_continues_on_error_if_specified(self):
        continue_on_error = True
        self.subject = runner.TaskRunner(self.config_mgr, self.registry, continue_on_error)
        self.given('multi_task_with_failures')
        self.expect_executed('task_2')


    def test_calls_tear_down_on_tasks_that_implement_it(self):
        self.given('has_tear_down')
        self.assertTrue(self.tear_down_task.tear_down_called)


    def test_does_not_call_teardown_if_script_fails_before_executing_task(self):
        with self.assertRaises(bterror.TaskError):
            self.given('partially_executed')
            self.assertFalse(self.tear_down_task.tear_down_called)


    def test_uses_same_execution_context_with_all_tasks(self):
        self.given('multi_task')
        self.assertTrue(self.context_shared)



    def given(self, task_name):
        try:
            self.subject.run(task_name)
        finally:
            self.subject.tear_down()


    def expect_executed(self, task_name):
        self.assertIn(task_name, self.executed_tasks)


    def task_1_callback(self, **kwargs):
        context = kwargs.get('context')
        context['shared-property'] = 1
        self.executed_tasks.append('task_1')


    def task_2_callback(self, **kwargs):
        config = kwargs.get('config')
        context = kwargs.get('context')
        if context.get('shared-property') == 1:
            self.context_shared = True
        self.executed_tasks.append('task_2')


    def invalid_task(self, noconfig):
        pass


    def configurable_task(self, **kwargs):
        config = kwargs.get('config')
        param = config.get('param')
        self.assertEqual(param, 'value')
        self.executed_tasks.append('configurable')


    def failing_task(self, **kwargs):
        raise bterror.TaskError()



class TearDownTask(object):

    def __init__(self):
        self.called = False
        self.tear_down_called = False
        return super(TearDownTask, self).__init__()

    def __call__(self, **kwargs):
        self.called = True

    def tear_down(self):
        self.tear_down_called = True



if __name__=='__main__':
    unittest.main()


"""
.. _task-pip:

pip
---

The ``pip`` task provides an automation hook to execute ``pip`` inside of
Bolt. In its simplest form, the task does not require any configuration,
and it just assumes a ``requirements.txt`` file is provided at the 
current working directory, which will be used to execute a ``pip install``.

The task also provides a simple form where a ``command`` and ``package``
are specified to allow install a single package. ::

    config = {
        'pip': {
            'command': 'install',
            'package': 'package_name'
        }
    }

The supported ``pip`` functionality can be configured by setting the
``command`` option to a valid ``pip`` command, and providing a set of
arguments to ``pip`` as an ``options`` dictionary where the keys are
valid ``pip`` arguments in short or long form without leading dashes
and the values are the respective argument values, or ``True`` in the
case of flags. The following shows a more advance use of this task. ::

    config = {
        'pip': {
            'command': 'install',
            'options': {
                'r': './data/project_requirements.txt',
                'target': './requirements',
                'upgrade': True,
                'force-reinstall': True
            }
        }
    }

"""
import logging
import pip
import bolt.errors as bterrors
import bolt.utils as utilities

DEFAULT_COMMAND = 'install'
DEFAULT_REQUIREMENTS_FILE = 'requirements.txt'
DEFAULT_ARGUMENTS = [DEFAULT_COMMAND, '-r', DEFAULT_REQUIREMENTS_FILE]

class PipError(bterrors.TaskError):

    def __init__(self, pip_code):
        msg = "pip exited with code: {code}".format(code=pip_code)
        super(PipError, self).__init__(msg)

class _PipArgumentGenerator(utilities.CommonCommandAndArgumentsGenerator):


    def __init__(self):
        return super(_PipArgumentGenerator, self).__init__(DEFAULT_ARGUMENTS)


    def _convert_config_to_arguments(self):
        self.command = self.config.get('command')
        self.package = self.config.get('package')
        if self._installing_single_package:
            self.args = [DEFAULT_COMMAND, self.package]
        else:
            super(_PipArgumentGenerator, self)._convert_config_to_arguments()
            

    @property
    def _installing_single_package(self):
        return self.command == DEFAULT_COMMAND and self.package



class ExecutePipTask(object):
    
    def __call__(self, **kwargs):
        logging.info('Executing Python Package Installer')
        config = kwargs.get('config')
        generator = _PipArgumentGenerator()
        self.args = generator.generate_from(config)
        logging.debug('Arguments: ' + repr(self.args))
        try:
            self._execute_pip()
        except SystemExit as exc:
            raise PipError(exc.code)
        return 0


    def _execute_pip(self):
        pip.main(self.args)




def register_tasks(registry):
    registry.register_task('pip', ExecutePipTask())
    


"""
conttest
--------

This task uses ``conttest`` to monitor a directory for changes and executes the specified
task everytime a change is made. The following configuration is supported::

    config = {
        'conttest': {
            'task': 'registered_task',
            'directory': './directory/to/monitor/'
        }
    }

The ``task`` parameter is the task to be executed and must be registered in ``boltfile.py``.
The ``directory`` parameter is the directory (including sub-directories) to monitor for 
changes.

To use this task, you need to have ``conttest`` installed, which you can do by calling::

    pip install conttest
"""
import logging
import os

import bolt 


class ExecuteConttest(object):

    def __call__(self, **kwargs):
        config = kwargs.get('config') 
        self.task_name = config.get('task')
        self.directory = config.get('directory') or os.getcwd()
        self.continue_on_error = True
        logging.info('Executing continously "{task}" at {directory}'.format(task=self.task_name, directory=self.directory))
        self.execute_task()


    def execute_task(self):
        import conttest.conttest as ct
        ct.watch_dir(self.directory, lambda: bolt.run_task(self.task_name, self.continue_on_error), method=ct.TIMES)



def register_tasks(registry):
    registry.register_task('conttest', ExecuteConttest())



from setuptools import setup, find_packages
import bolt.about

name = bolt.about.project
version = bolt.about.release
description = 	bolt.about.description
author = bolt.about.author

packages = find_packages()
entry_points = {
    'console_scripts': [
        'bolt = bolt:run'
    ]
}

setup(name=name,
      version=version,
      description=description,
      author=author,
      packages=packages,
      entry_points=entry_points)



"""
shell
-----

The ``shell`` task allows executing a shell command with specified arguments
inside the bolt execution context. This task comes handy when no bolt
specific implementation has been provided for a particular task or to invoke
an existing script that should be included as part of the process. 

The trade-off of using this task is that commands are system specific and
it makes it harder to implement a cross-platform ``boltfile.py``.

The task takes a ``command`` parameter specifying the command to be executed,
and an ``arguments`` option that must be set to a list of string for each of
the command line argument tokens to be passed to the tool.

The following example shows how to invoke an existing |python|_ script that
takes a few parameters::

    config = {
        'shell': {
            'command': 'python',
            'arguments': ['existing_script.py', '--with-argument', '-f', '--arg-with', 'a_value']
        }
    }

..  todo::  Find a better example.
"""
import logging
import subprocess as sp

import bolt.errors as bterror
from bolt.errors import InvalidConfigurationError

class ShellError(bterror.TaskError):

    def __init__(self, shell_code):
        msg = "shell command exited with code {code}".format(code=shell_code)
        super(ShellError, self).__init__(msg)


class ShellExecuteTask(object):
    
    def __call__(self, **kwargs):
        self.config = kwargs.get('config')
        self._verify_valid_configuration()
        self._build_command_line()
        self._run()


    def _verify_valid_configuration(self):
        if not self.config:
            raise InvalidConfigurationError('A shell command is required')
        self.command = self.config.get('command')
        if not self.command:
            raise InvalidConfigurationError('A command must be specified')


    def _build_command_line(self):
        self.command_line = [self.command]
        arguments = self.config.get('arguments')
        if arguments:
            self.command_line.extend(arguments)


    def _run(self):
        logging.debug('Shell command line: ', repr(self.command_line))
        result = sp.call(self.command_line)
        if result != 0:
            raise ShellError(result)

        


def register_tasks(registry):
    registry.register_task('shell', ShellExecuteTask())



"""
"""
import logging

from bolt.errors import InvalidTaskError

class TaskRegistry(object):
    """

    """
    def __init__(self):
        self._tasks = {}

    def register_task(self, name, task):
        """

        :param name:
        :param task:
        :return:
        """
        if not self._is_valid_task(task):
            raise InvalidTaskError()
        self._tasks[name] = task
        msg = '{task_name} task is registered.'.format(task_name=name)
        logging.debug(msg)


    def register_module_tasks(self, module):
        """
        """
        module.register_tasks(self)


    def get(self, name):
        """

        :param name:
        :return:
        """
        task_name = self._extract_task_name(name)
        return  self._tasks[task_name]


    def _is_valid_task(self, task):
        is_list = isinstance(task, list)
        if is_list:
            for t in task:
                if not isinstance(t, str):
                    return False
        return callable(task) or is_list


    def _extract_task_name(self, full_task):
        return full_task.split('.')[0]










import unittest

import bolt._btconfig as config


class TestConfigurationManager(unittest.TestCase):

    def setUp(self):
        self.configuration = {
            'task': {
                'option': 1,
                'overridable': 'initial',
                'child': {
                    'overridable': 'inchild',
                    'grandchild': {
                        'overridable': 'ingrandchild'
                    }
                }
            }
        }
        self.subject = config.ConfigurationManager(self.configuration)


    def test_returns_empty_configuration_if_task_id_does_not_exists(self):
        config = self.subject.get('inexistent')
        self.assertFalse(config)


    def test_returns_the_specified_configuration(self):
        self.given('task')
        self.expect('option', 1)


    def test_options_can_be_overriden_in_child_configurations(self):
        self.given('task.child')
        self.expect('overridable', 'inchild')


    def test_child_configuration_maintains_parent_values_if_not_overriden(self):
        self.given('task.child')
        self.expect('option', 1)


    def test_supports_many_levels_of_child_indentation(self):
        self.given('task.child.grandchild')
        self.expect('overridable', 'ingrandchild')


    def given(self, key):
        self.result = self.subject.get(key)


    def expect(self, option, expected):
        actual_value = self.result.get(option)
        self.assertEqual(actual_value, expected)




if __name__=="__main__":
    unittest.main()



"""

"""

class ConfigurationManager(object):

    def __init__(self, full_config):
        self._full_config = full_config


    def get(self, task_key):
        self.result_config = {}
        self.current_config = self._full_config.copy()
        [self._add_task_options(task_option) for task_option in  task_key.split('.')]
        return self.result_config
        
        
    def _add_task_options(self, task_option):
        task_config = self.current_config.get(task_option)
        if task_config:
            self._merge_into_result(task_config)
            self._remove_from_result(task_option)
        self.current_config = task_config
        
        
    def _merge_into_result(self, task_config):
        self.result_config.update(task_config)
        
        
    def _remove_from_result(self, task_option):
        if task_option in self.result_config:
            del self.result_config[task_option]
        
